'use strict';
const http = require('http');
const https = require('https');
const httpResolveALPN = require('./utils/http-resolve-alpn');
const urlToOptions = require('./utils/url-to-options');
const HTTP2ClientRequest = require('./client-request');

module.exports = async (input, options) => {
	if (typeof input === 'string' || input instanceof URL) {
		input = urlToOptions(new URL(input));
	}

	options = {...input, ...options};
	options.protocol = options.protocol || 'https:';
	options.port = options.port || (options.protocol === 'http:' ? 80 : 443);

	if (options.protocol === 'https:') {
		const {alpnProtocol, socket} = await httpResolveALPN({
			...options,
			resolveSocket: !options.createConnection
		});

		if (socket) {
			options.createConnection = () => socket;
		}

		if (alpnProtocol === 'h2') {
			return new HTTP2ClientRequest(options);
		}

		options._defaultAgent = https.globalAgent;
	}

	options.session = options.socketSession;
	return new http.ClientRequest(options);
};

module.exports.resolveALPN = httpResolveALPN;
